#include<stdio.h>
#include<stdlib.h>
#include<math.h>

#define N 20			// # of PF inputs p.273 Fig 1
#define Ntrials 50		// # of repeated presentations
#define T 1.0			// 1 s
#define DT 0.001		// 1 ms
#define eta 0.1			// learning rate
#define delta 0.1		// time window for delay learning
#define tau_m ((0.01/(DT)))	// membrane time constant
#define tau_s ((0.00005/(DT)))	// spectral time constant
#define voltage_filename "V.dat"
#define spectra_filename "s.dat"

// Eq. (4)
double D(const double dt)
{
  if (-delta <= dt && dt <= delta){
    return dt;
  }else{
    return 0;
  }
}

// postsynaptic delay d was distributed uniformly.
void initialize(double p[], double d[], double d_init[])
{
  int j;
  for(j = 0; j < N; j++){
    p[j] = 0;
  }
  for(j = 0; j < N; j++){
    d[j] = T/N*j;
    d_init[j] = d[j];
  }
}

// plotting the spectra of Ca2+-dependent K+ currents.
void spectra(const double d[])
{
  FILE *file;
  int j;
  double t, I;

  file = fopen(spectra_filename, "w");
  for(j = 0; j < N; j++){
    t = 0;
    while(t < T){
      if (t > d[j]){
	I = ((d[j]-t)/tau_s) * exp(1-(t-d[j])/tau_s);
      }else{
	I = 0;
      }
      fprintf(file, "%f %f\n", t, I);
      t += DT;
    }
    fprintf(file, "\n\n");
  }
}

void loop(const double t_CF)	// t_CF is the time of CF signal onset.
{
  double t, dV, V;
  double I[N], p[N], d[N];
  double d_init[N], I_init[N];
  int trial, j;
  FILE *file;

  initialize(p, d, d_init);
  spectra(d);

  file = fopen(voltage_filename, "w");
  for(trial = 0; trial < Ntrials; trial++){
    t = 0;
    V = 0;
    while(t < T){
      
      // Eq. (2)
      for(j = 0; j < N; j++){
	if (t > d[j]){
	  I[j] = ((d[j]-t)/tau_s) * exp(1-(t-d[j])/tau_s);
	  I_init[j] = ((d_init[j]-t)/tau_s) * exp(1-(t-d_init[j])/tau_s);
	}else{
	  I[j] = 0;
	  I_init[j] = 0;
	}
      }
      // Eq. (1)
      dV = -V/tau_m;
      for(j = 0; j < N; j++){
	dV += (I[j] - I_init[j])/N;
      }
      V += dV;

      fprintf(file, "%f %f\n", t, V);
      t += DT;
    }

    // Eq. (3)
    for(j = 0; j < N; j++){
      d[j] += eta * D(t_CF - d[j]);
    }

    fprintf(file, "\n\n");
  }
  fclose(file);
}

int main(void)
{
  double t_CF;

  t_CF = 0.5;

  loop(t_CF);

  return 0;
}
